#ifndef __XMMSC_XMMSC_GLIB_H__
#define __XMMSC_XMMSC_GLIB_H__

#include "xmmsc/xmmsc_idnumbers.h"

void xmmsc_log_glib_handler (const char *, xmmsc_log_level_t, const char *, void *);

#endif /* __XMMSC_XMMSC_GLIB_H__ */
